@@ -7,17 +7,38 @@ import numpy as np
 
 from ainshamsflow import activations
 from ainshamsflow import initializers
+from ainshamsflow.utils.utils import get_indices, col2im, im2col
 from ainshamsflow.utils.asf_errors import (BaseClassError, NameNotFoundError, UnsupportedShapeError,
 										   InvalidShapeError, WrongObjectError, InvalidPreceedingLayerError,
 										   InvalidRangeError)
 
 
+__pdoc__ = dict()
+
+__pdoc__['Layer.add_input_shape_to_layer'] = False
+__pdoc__['Layer.diff'] = False
+__pdoc__['Layer.count_params'] = False
+__pdoc__['Layer.get_weights'] = False
+__pdoc__['Layer.set_weights'] = False
+
+for layer_n in ['Dense', 'BatchNorm', 'Dropout',
+				'Conv1D', 'Pool1D', 'GlobalPool1D', 'Upsample1D',
+				'Conv2D', 'Pool2D', 'GlobalPool2D', 'Upsample2D',
+				'FastConv2D', 'FastPool2D',
+				'Conv3D', 'Pool3D', 'GlobalPool3D', 'Upsample3D',
+				'Flatten', 'Activation', 'Reshape']:
+	__pdoc__[layer_n + '.__call__'] = True
+	__pdoc__[layer_n + '.diff'] = True
+	__pdoc__[layer_n + '.add_input_shape_to_layer'] = False
+
+
 def get(layer_name):
 	"""Get any Layer in this Module by name."""
 
 	layers = [Dense, BatchNorm, Dropout,
 			  Conv1D, Pool1D, GlobalPool1D, Upsample1D,
-			  Conv2D, Pool2D, GlobalPool2D,  Upsample2D,
+			  Conv2D, Pool2D, GlobalPool2D, Upsample2D,
+			  FastConv2D, FastPool2D,
 			  Conv3D, Pool3D, GlobalPool3D, Upsample3D,
 			  Flatten, Activation, Reshape]
 	for layer in layers:
@@ -32,14 +53,18 @@ class Layer:
 
 	To create a new Layer, create a class that inherits from this class.
 	You then have to add any parameters in your constructor
-	(while still calling this class' constructor) and redefine the
-	__call__(), diff(), add_input_shape_to_layer(), (Manditory)
+	(while still calling this class' constructor) and redefine the \_\_call\_\_(),
+	diff(), add_input_shape_to_layer(), (Manditory)
 	count_params(), get_weights() and set_weights() (Optional)
 	methods.
 	"""
 
 	def __init__(self, name=None, trainable=False):
-		"""Initialize the name and trainable parameter of the layer."""
+		"""
+		Args:
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 
 		if not isinstance(trainable, bool):
 			raise WrongObjectError(trainable, True)
@@ -92,6 +117,16 @@ class Dense(Layer):
 	def __init__(self, n_out, activation=activations.Linear(),
 				 weights_init=initializers.Normal(), biases_init=initializers.Constant(0),
 				 trainable=True, name=None):
+		"""
+		Activations and Initializers can be strings or objects.
+		Args:
+			n_out: number of (output) neurons in this layer.
+			activation: activation function used for the layer.
+			weights_init: Initializer for the weights of this layer.
+			biases_init: Initializer for the biases of this layers.
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if not isinstance(n_out, int):
 			raise WrongObjectError(n_out, 1)
 		if n_out <= 0:
@@ -120,7 +155,7 @@ class Dense(Layer):
 		self.x = None
 		self.z = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 1:
 			raise InvalidPreceedingLayerError(self)
 
@@ -175,6 +210,17 @@ class BatchNorm(Layer):
 				 gamma_init=initializers.Constant(1), beta_init=initializers.Constant(0),
 				 mu_init=initializers.Constant(0), sig_init=initializers.Constant(1),
 				 trainable=True, name=None):
+		"""
+		Args:
+			epsilon:
+			momentum:
+			gamma_init: Initializer for the weights of this layer.
+			beta_init: Initializer for the biases of this layer.
+			mu_init: Initializer for the means of this layer.
+			sig_init: Initializer for the standard deviations of this layer.
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if not isinstance(epsilon, float):
 			raise WrongObjectError(epsilon, float())
 		if not 0 < epsilon < 1:
@@ -211,7 +257,7 @@ class BatchNorm(Layer):
 
 		self.x_norm = None
 
-	def add_input_shape_to_layers(self, n):
+	def add_input_shape_to_layer(self, n):
 		self.n_out = (1,) + n
 		self.input_shape = self.output_shape = '(None' + (',{:4}' * len(n)).format(*n) + ')'
 
@@ -277,6 +323,11 @@ class Dropout(Layer):
 	__name__ = 'Dropout'
 
 	def __init__(self, prob, name=None):
+		"""
+		Args:
+			 prob: probability of keeping a neuron.
+			 name: name of  the layer.
+		"""
 		if not 0 < prob <= 1:
 			raise InvalidRangeError(prob, 0, 1)
 		self.rate = prob
@@ -288,7 +339,7 @@ class Dropout(Layer):
 		self.output_shape = None
 		self.filters = None
 
-	def add_input_shape_to_layers(self, n):
+	def add_input_shape_to_layer(self, n):
 		self.n_out = self.n_in = n
 		self.input_shape = self.output_shape = '(None' + (',{:4}'*len(n)).format(*n) + ')'
 		return n
@@ -315,6 +366,25 @@ class Conv1D(Layer):
 	def __init__(self, filters, kernel_size, strides=1, padding='valid',
 				 kernel_init=initializers.Normal(), biases_init=initializers.Constant(0),
 				 activation=activations.Linear(), name=None, trainable=True):
+		"""
+		Args:
+			filters: number of filters (kernels)
+			kernel_size: An integer or tuple of 1 integer, specifying the height and
+				width of the 1D convolution window. Can be a single integer to specify
+				the same value for all spatial dimensions.
+			strides: An integer or tuple of 1 integer, specifying the strides of the
+				convolution along the height and width. Can be a single integer to
+				specify the same value for all spatial dimensions. Specifying any stride
+				value != 1 is incompatible with specifying any dilation_rate value != 1.
+			padding: one of "valid" or "same" (case-insensitive). "valid" means no padding.
+				"same" results in padding evenly to the left/right or up/down of the input such
+				that output has the same height/width dimension as the input.
+			kernel_init: Initializer for the weights of this layer.
+			biases_init: Initializer for the biases of this layer.
+			activation: activation function used for the layer.
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if not isinstance(filters, int):
 			raise WrongObjectError(filters, 0)
 		if isinstance(kernel_size, int):
@@ -362,7 +432,7 @@ class Conv1D(Layer):
 		self.kernel = kernel_size
 		self.biases = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 2:
 			raise InvalidPreceedingLayerError(self)
 
@@ -467,6 +537,20 @@ class Pool1D(Layer):
 	__name__ = 'Pool1D'
 
 	def __init__(self, pool_size=2, strides=None, padding='valid', mode='max', name=None, trainable=True):
+		"""
+		Args:
+			pool_size: Integer or tuple of 1 integer. Factor by which to downscale. (2,) will halve
+			the input dimension. If only one integer is specified, the same window length will be used
+			for all dimensions.
+			strides: Integer, or tuple of 1 integer. Factor by which to downscale. E.g. 2 will halve the input.
+				If None, it will default to pool_size.
+			padding: One of "valid" or "same" (case-insensitive). "valid" means no padding. "same"
+				results in padding evenly to the left/right or up/down of the input such that output
+				has the same height/width dimension as the input.
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if isinstance(pool_size, int):
 			pool_size = (pool_size,)
 		if not isinstance(pool_size, tuple):
@@ -511,7 +595,7 @@ class Pool1D(Layer):
 		self.x = None
 		self.z = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 2:
 			raise InvalidPreceedingLayerError(self)
 
@@ -593,6 +677,11 @@ class GlobalPool1D(Layer):
 	__name__ = 'GlobalPool1D'
 
 	def __init__(self, mode='max', name=None):
+		"""
+		Args:
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+		"""
 		if not (mode == 'max' or mode == 'avg' or mode == 'average' or mode == 'mean'):
 			raise InvalidRangeError(mode, 'max', 'avg')
 		if mode == 'max':
@@ -649,6 +738,11 @@ class Upsample1D(Layer):
 	__name__ = 'Upsample1D'
 
 	def __init__(self, size=2, name=None):
+		"""
+		Args:
+			size: Int, or tuple of 1 integer. The upsampling factors for rows and columns.
+			name: name of  the layer.
+		"""
 		if isinstance(size, int):
 			size = (size,)
 		if not isinstance(size, tuple):
@@ -667,7 +761,7 @@ class Upsample1D(Layer):
 		self.input_shape = None
 		self.output_shape = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 2:
 			raise InvalidPreceedingLayerError(self)
 
@@ -711,6 +805,25 @@ class Conv2D(Layer):
 	def __init__(self, filters, kernel_size, strides=1, padding='valid',
 				 kernel_init=initializers.Normal(), biases_init=initializers.Constant(0),
 				 activation=activations.Linear(), name=None, trainable=True):
+		"""
+		Args:
+			filters: number of filters (kernels)
+			kernel_size: An integer or tuple of 2 integers, specifying the height and
+				width of the 2D convolution window. Can be a single integer to specify
+				the same value for all spatial dimensions.
+			strides: An integer or tuple of 2 integers, specifying the strides of the
+				convolution along the height and width. Can be a single integer to
+				specify the same value for all spatial dimensions. Specifying any stride
+				value != 1 is incompatible with specifying any dilation_rate value != 1.
+			padding: one of "valid" or "same" (case-insensitive). "valid" means no padding.
+				"same" results in padding evenly to the left/right or up/down of the input such
+				that output has the same height/width dimension as the input.
+			kernel_init: Initializer for the weights of this layer.
+			biases_init: Initializer for the biases of this layer.
+			activation: activation function used for the layer.
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if not isinstance(filters, int):
 			raise WrongObjectError(filters, 0)
 		if isinstance(kernel_size, int):
@@ -758,7 +871,7 @@ class Conv2D(Layer):
 		self.kernel = kernel_size
 		self.biases = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 3:
 			raise InvalidPreceedingLayerError(self)
 
@@ -866,12 +979,111 @@ class Conv2D(Layer):
 		self.biases = np.array(biases)
 
 
+class FastConv2D(Conv2D):
+	"""Efficient 2-Dimensional Convolution Layer."""
+
+	__name__ = 'FastConv2D'
+
+	def __init__(self, *args, **kwargs):
+		super().__init__(*args, **kwargs)
+		self.idx = None
+
+	def add_input_shape_to_layer(self, n_in):
+		n_out = super().add_input_shape_to_layer(n_in)
+		f_h, f_w = self.kernel.shape[1:3]
+		p_h = (self.kernel[0] - 1) // 2 if self.same_padding else 0
+		p_w = (self.kernel[1] - 1) // 2 if self.same_padding else 0
+		x_shape = (1,) + n_in
+
+		self.idx = get_indices(x_shape, f_h, f_w, self.strides, (p_h, p_w)) # i, j ,d
+
+		return n_out
+
+	def __call__(self, x, training=False):
+		x = x.transpose(0, 3, 1, 2)
+		kernel = self.kernel.transpose(0, 3, 1, 2)
+
+		m, n_C_prev, n_H_prev, n_W_prev = x.shape
+
+		n_c_out, f_h, f_w, n_c_in = self.kernel.shape
+		p_h, p_w = 0, 0
+		s_h, s_w = self.strides
+		if self.same_padding:
+			p_h, p_w = (f_h - 1) // 2, (f_w - 1) // 2
+
+		n_H = int((n_H_prev + 2 * p_h - f_h) / s_h) + 1
+		n_W = int((n_W_prev + 2 * p_w - f_w) / s_w) + 1
+
+		X_col = im2col(x, self.idx, (p_h, p_w))
+		w_col = kernel.reshape((n_c_out, -1))
+		b_col = self.biases.reshape(-1, 1)
+
+		# Perform matrix multiplication.
+		out = w_col @ X_col + b_col
+
+		# Reshape back matrix to image.
+		out = np.array(np.hsplit(out, m)).reshape((m, n_c_out, n_H, n_W))
+		out = out.transpose(0, 2, 3, 1)
+
+		self.cache = x, X_col, w_col
+		return out
+
+	def diff(self, da):
+		da = da.transpose(0, 3, 1, 2)
+
+		n_c_out, f_h, f_w, n_c_in = self.kernel.shape
+		p_h, p_w = 0, 0
+		if self.same_padding:
+			p_h, p_w = (f_h - 1) // 2, (f_w - 1) // 2
+
+		x, X_col, w_col = self.cache
+		m = x.shape[0]
+
+		# Compute bias gradient.
+		db = np.sum(da, axis=(0, 2, 3))
+
+		# Reshape da properly.
+		da = da.reshape(da.shape[0] * da.shape[1], da.shape[2] * da.shape[3])
+		da = np.array(np.vsplit(da, m))
+		da = np.concatenate(da, axis=-1)
+
+		# Perform matrix multiplication between reshaped da and w_col to get dX_col.
+		dX_col = w_col.T @ da
+		# Perform matrix multiplication between reshaped da and X_col to get dW_col.
+		dw_col = da @ X_col.T
+
+		# Reshape back to image (col2im).
+		dX = col2im(dX_col, x.shape, self.idx, (p_h, p_w))
+		# Reshape dw_col into dw.
+		dW = dw_col.reshape((n_c_out, n_c_in, f_h, f_w))
+
+		dX = dX.transpose(0, 2, 3, 1)
+		dW = dW.transpose(0, 2, 3, 1)
+		db = db.reshape((-1, 1, 1, 1))
+
+		return dX, dW, db
+
+
 class Pool2D(Layer):
 	"""2-Dimensional Pooling Layer."""
 
 	__name__ = 'Pool2D'
 
 	def __init__(self, pool_size=2, strides=None, padding='valid', mode='max', name=None, trainable=True):
+		"""
+		Args:
+			pool_size: Integer or tuple of 2 integers, factors by which to downscale. (2, 2) will halve
+			the input dimensions. If only one integer is specified, the same window length will be used
+			for all dimensions.
+			strides: Integer, or tuple of 2 integers. Factor by which to downscale. 2 will halve the input.
+				If None, it will default to pool_size.
+			padding: One of "valid" or "same" (case-insensitive). "valid" means no padding. "same"
+				results in padding evenly to the left/right or up/down of the input such that output
+				has the same height/width dimension as the input.
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if isinstance(pool_size, int):
 			pool_size = (pool_size, pool_size)
 		if not isinstance(pool_size, tuple):
@@ -916,7 +1128,7 @@ class Pool2D(Layer):
 		self.x = None
 		self.z = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 3:
 			raise InvalidPreceedingLayerError(self)
 
@@ -1001,12 +1213,89 @@ class Pool2D(Layer):
 		return dx, np.array([[0]]), np.array([[0]])
 
 
+class FastPool2D(Pool2D):
+	"""Efficient 2-Dimensional Pooling Layer."""
+
+	__name__ = 'FastPool2D'
+
+	def __init__(self, *args, **kwargs):
+		super().__init__(*args, **kwargs)
+		self.idx = None
+
+	def add_input_shape_to_layer(self, n_in):
+		n_out = super().add_input_shape_to_layer(n_in)
+		f_h, f_w = self.pool_size
+		p_h = (self.pool_size[0] - 1) // 2 if self.same_padding else 0
+		p_w = (self.pool_size[1] - 1) // 2 if self.same_padding else 0
+		x_shape = (1,) + n_in
+
+		self.idx = get_indices(x_shape, f_h, f_w, self.strides, (p_h, p_w))  # i, j ,d
+
+		return n_out
+
+	def __call__(self, x, training=False):
+		self.x = x.transpose(0, 3, 1, 2)
+
+		m, n_C_prev, n_H_prev, n_W_prev = self.x.shape
+		f_h, f_w = self.pool_size
+		p_h, p_w = 0, 0
+		s_h, s_w = self.strides
+		if self.same_padding:
+			p_h, p_w = (f_h - 1) // 2, (f_w - 1) // 2
+
+		n_C = n_C_prev
+		n_H = int((n_H_prev + 2 * p_h - f_h) / s_h) + 1
+		n_W = int((n_W_prev + 2 * p_w - f_w) / s_w) + 1
+
+		X_col = im2col(self.x, self.idx, (p_h, p_w))
+		X_col = X_col.reshape(n_C, X_col.shape[0] // n_C, -1)
+		if self.mode == 'max':
+			A_pool = np.max(X_col, axis=1)
+			# self.mask = np.equal()
+		else:
+			A_pool = np.mean(X_col, axis=1)
+		# Reshape A_pool properly.
+		A_pool = np.array(np.hsplit(A_pool, m))
+		A_pool = A_pool.reshape((m, n_C, n_H, n_W))
+
+		A_pool = A_pool.transpose(0, 2, 3, 1)
+		return A_pool
+
+	def diff(self, da):
+		dout = da.transpose(0, 3, 1, 2)
+
+		m, n_C_prev, n_H_prev, n_W_prev = self.x.shape
+		f_h, f_w = self.pool_size
+		p_h, p_w = 0, 0
+		s_h, s_w = self.strides
+		if self.same_padding :
+			p_h, p_w = (f_h - 1) // 2, (f_w - 1) // 2
+
+		n_C = n_C_prev
+
+		dout_flatten = dout.reshape(n_C, -1) / (f_h * f_w)
+		dX_col = np.repeat(dout_flatten, f_h * f_w, axis=0)
+		dX = col2im(dX_col, self.x.shape, self.idx, (p_h, p_w))
+		# Reshape dX properly.
+		dX = dX.reshape(m, -1)
+		dX = np.array(np.hsplit(dX, n_C_prev))
+		dX = dX.reshape((m, n_C_prev, n_H_prev, n_W_prev))
+
+		dX = dX.transpose(0, 2, 3, 1)
+		return dX, np.array([[0]]), np.array([[0]])
+
+
 class GlobalPool2D(Layer):
 	"""Global Pooling Layer."""
 
 	__name__ = 'GlobalPool2D'
 
 	def __init__(self, mode='max', name=None):
+		"""
+		Args:
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+		"""
 		if not (mode == 'max' or mode == 'avg' or mode == 'average' or mode == 'mean'):
 			raise InvalidRangeError(mode, 'max', 'avg')
 		if mode == 'max':
@@ -1063,6 +1352,11 @@ class Upsample2D(Layer):
 	__name__ = 'Upsample2D'
 
 	def __init__(self, size=2, name=None):
+		"""
+		Args:
+			size: Int, or tuple of 2 integers. The upsampling factors for rows and columns.
+			name: name of  the layer.
+		"""
 		if isinstance(size, int):
 			size = (size, size)
 		if not isinstance(size, tuple):
@@ -1081,7 +1375,7 @@ class Upsample2D(Layer):
 		self.input_shape = None
 		self.output_shape = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 3:
 			raise InvalidPreceedingLayerError(self)
 
@@ -1128,6 +1422,25 @@ class Conv3D(Layer):
 	def __init__(self, filters, kernel_size, strides=1, padding='valid',
 				 kernel_init=initializers.Normal(), biases_init=initializers.Constant(0),
 				 activation=activations.Linear(), name=None, trainable=True):
+		"""
+		Args:
+			filters: number of filters (kernels)
+			kernel_size: An integer or tuple of 3 integers, specifying the height and
+				width of the 3D convolution window. Can be a single integer to specify
+				the same value for all spatial dimensions.
+			strides: An integer or tuple of 3 integers, specifying the strides of the
+				convolution along the height and width. Can be a single integer to
+				specify the same value for all spatial dimensions. Specifying any stride
+				value != 1 is incompatible with specifying any dilation_rate value != 1.
+			padding: one of "valid" or "same" (case-insensitive). "valid" means no padding.
+				"same" results in padding evenly to the left/right or up/down of the input such
+				that output has the same height/width dimension as the input.
+			kernel_init: Initializer for the weights of this layer.
+			biases_init: Initializer for the biases of this layer.
+			activation: activation function used for the layer.
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if not isinstance(filters, int):
 			raise WrongObjectError(filters, 0)
 		if isinstance(kernel_size, int):
@@ -1175,7 +1488,7 @@ class Conv3D(Layer):
 		self.kernel = kernel_size
 		self.biases = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 4:
 			raise InvalidPreceedingLayerError(self)
 
@@ -1299,6 +1612,20 @@ class Pool3D(Layer):
 	__name__ = 'Pool3D'
 
 	def __init__(self, pool_size=2, strides=None, padding='valid', mode='max', name=None, trainable=True):
+		"""
+		Args:
+			pool_size: Integer or tuple of 3 integers, factors by which to downscale. (2, 2, 2) will halve
+			the input dimensions. If only one integer is specified, the same window length will be used
+			for all dimensions.
+			strides: Integer, or tuple of 3 integers. Factor by which to downscale. 2 will halve the input.
+				If None, it will default to pool_size.
+			padding: One of "valid" or "same" (case-insensitive). "valid" means no padding. "same"
+				results in padding evenly to the left/right or up/down of the input such that output
+				has the same height/width dimension as the input.
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+			trainable: Boolean to define whether this layer is trainable or not.
+		"""
 		if isinstance(pool_size, int):
 			pool_size = (pool_size, pool_size, pool_size)
 		if not isinstance(pool_size, tuple):
@@ -1343,7 +1670,7 @@ class Pool3D(Layer):
 		self.x = None
 		self.z = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 4:
 			raise InvalidPreceedingLayerError(self)
 
@@ -1445,6 +1772,11 @@ class GlobalPool3D(Layer):
 	__name__ = 'GlobalPool3D'
 
 	def __init__(self, mode='max', name=None):
+		"""
+		Args:
+			mode: One of "max" or "avg" (case-insentitive).
+			name: name of  the layer.
+		"""
 		if not (mode == 'max' or mode == 'avg' or mode == 'average' or mode == 'mean'):
 			raise InvalidRangeError(mode, 'max', 'avg')
 		if mode == 'max':
@@ -1501,6 +1833,11 @@ class Upsample3D(Layer):
 	__name__ = 'Upsample3D'
 
 	def __init__(self, size=2, name=None):
+		"""
+		Args:
+			size: Int, or tuple of 2 integers. The upsampling factors for rows and columns.
+			name: name of  the layer.
+		"""
 		if isinstance(size, int):
 			size = (size, size, size)
 		if not isinstance(size, tuple):
@@ -1519,7 +1856,7 @@ class Upsample3D(Layer):
 		self.input_shape = None
 		self.output_shape = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		if len(n_in) != 4:
 			raise InvalidPreceedingLayerError(self)
 
@@ -1576,13 +1913,17 @@ class Flatten(Layer):
 	__name__ = 'Flatten'
 
 	def __init__(self, name=None):
+		"""
+		Args:
+			name: name of  the layer.
+		"""
 		super().__init__(name, False)
 		self.n_out = None
 		self.n_in = None
 		self.input_shape = None
 		self.output_shape = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		self.n_in = n_in
 		self.n_out = (np.prod(n_in),)
 		self.input_shape = '(None' + (',{:4}'*len(self.n_in)).format(*self.n_in) + ')'
@@ -1606,6 +1947,11 @@ class Activation(Layer):
 	__name__ = 'Activation'
 
 	def __init__(self, act, name=None):
+		"""
+		Args:
+			act: Either an activation instance or string.
+			name: name of  the layer.
+		"""
 		if isinstance(act, str):
 			self.activation = activations.get(act)
 		if not isinstance(act, activations.Activation()):
@@ -1624,7 +1970,7 @@ class Activation(Layer):
 		self.output_shape = None
 		self.x = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		self.n_in = self.n_out = n_in
 		self.input_shape = self.output_shape = '(None' + (',{:4}' * len(n_in)).format(*n_in) + ')'
 		return self.n_out
@@ -1650,6 +1996,10 @@ class Reshape(Layer):
 	__name__ = 'Reshape'
 
 	def __init__(self, n_out, name=None):
+		"""
+		Args:
+			name: name of  the layer.
+		"""
 		if not isinstance(n_out, tuple):
 			raise WrongObjectError(n_out, tuple())
 
@@ -1671,7 +2021,7 @@ class Reshape(Layer):
 		self.input_shape = None
 		self.output_shape = None
 
-	def add_input_shape_to_layers(self, n_in):
+	def add_input_shape_to_layer(self, n_in):
 		self.n_in = n_in
 
 		if self.unk_ch_id is not None:
